using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Activities;
using Microsoft.TeamFoundation.Build.Client;
using Microsoft.TeamFoundation.Build.Workflow.Activities;
using System.IO;

namespace Benday.TeamBuild2012.Activities
{
    [BuildActivity(HostEnvironmentOption.Agent)]
    public sealed class MoveFile : BuildActivityBase
    {
        public override string ActivityName
        {
            get { return "MoveFile"; }
        }

        [RequiredArgument]
        public InArgument<string> SourcePath { get; set; }

        [RequiredArgument]
        public InArgument<string> TargetPath { get; set; }

        protected override void Execute(CodeActivityContext context)
        {
            try
            {
                LogDebugMessage(context, "Initializing...");

                string tempSourceFile = SourcePath.Get(context);
                string tempTargetFile = TargetPath.Get(context);

                LogDebugVariableValue(context, "SourcePath", tempSourceFile);
                LogDebugVariableValue(context, "TargetPath", tempTargetFile);

                if (File.Exists(tempSourceFile) == false)
                {
                    throw new InvalidOperationException(String.Format("Source file ('{0}') does not exist.", tempSourceFile));
                }

                if (File.Exists(tempTargetFile) == true)
                {
                    LogDebugMessage(context, "Target file exists.  Setting read only flag to false and deleting.");

                    File.SetAttributes(tempTargetFile, FileAttributes.Normal);
                    File.Delete(tempTargetFile);
                }

                LogDebugMessage(context, "Calling move...");
                File.Move(tempSourceFile, tempTargetFile);

                LogUsefulMessage(context, 
                        String.Format("Moved file from '{0}' to '{1}'.",
                            tempSourceFile, tempTargetFile));
            }
            catch (Exception ex)
            {
                LogException(context, ex);
            }
        }
    }
}
