using System;
using System.Collections.Generic;
using System.Linq;
using System.Activities;
using Microsoft.TeamFoundation.Build.Client;
using System.ComponentModel;

namespace Benday.TeamBuild2012.Activities
{
    [BuildActivity(HostEnvironmentOption.Agent)]
    public sealed class UpdateAspNetSessionState : BuildActivityBase
    {
        public override string ActivityName
        {
            get { return "UpdateAspNetSessionState"; }
        }

        [RequiredArgument]
        public InArgument<string> FileName { get; set; }

        [DefaultValue(null)] 
        public InArgument<string> Mode { get; set; }

        [DefaultValue(null)]
        public InArgument<string> StateConnectionString { get; set; }

        [DefaultValue(null)]
        public InArgument<string> SqlConnectionString { get; set; }

        [DefaultValue(null)]
        public InArgument<string> AllowCustomSqlDatabase { get; set; }

        [DefaultValue(null)]
        public InArgument<string> Cookieless { get; set; }

        [DefaultValue(null)]
        public InArgument<string> Timeout { get; set; }

        protected override void Execute(CodeActivityContext context)
        {
            try
            {
                LogDebugMessage(context, "Initializing...");

                string tempFileName = FileName.Get(context);
                string tempMode = Mode.Get(context);
                string tempStateConnectionString = StateConnectionString.Get(context);
                string tempSqlConnectionString = SqlConnectionString.Get(context);
                string tempAllowCustomSqlDatabase = AllowCustomSqlDatabase.Get(context);
                string tempCookieless = Cookieless.Get(context);
                string tempTimeout = Timeout.Get(context);

                LogDebugVariableValue(context, "FileName", tempFileName);
                LogDebugVariableValue(context, "Mode", tempMode);
                LogDebugVariableValue(context, "StateConnectionString", tempStateConnectionString);
                LogDebugVariableValue(context, "SqlConnectionString", tempSqlConnectionString);
                LogDebugVariableValue(context, "AllowCustomSqlDatabase", tempAllowCustomSqlDatabase);
                LogDebugVariableValue(context, "Cookieless", tempCookieless);
                LogDebugVariableValue(context, "Timeout", tempTimeout);

                LogDebugMessage(context, "Calling SetSessionStateConfiguration()...");

                WebConfigEditor.SetSessionStateConfiguration(
                    tempFileName, 
                    tempMode, 
                    tempStateConnectionString, 
                    tempSqlConnectionString, 
                    tempAllowCustomSqlDatabase, 
                    tempCookieless, 
                    tempTimeout);

                LogUsefulMessage(context, String.Format("Updated session state configuration for '{0}'.", tempFileName));
            }
            catch (Exception ex)
            {
                LogException(context, ex);

                throw;
            }
        }
    }
}
