using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using System.IO;

namespace Benday.TeamBuild2012.Activities
{
    public class WebConfigEditor
    {
        public static void SetSessionStateConfiguration(string filename,
            string mode,
            string stateConnectionString,
            string sqlConnectionString,
            string allowCustomSqlDatabase,
            string cookieless,
            string timeout)
        {
            if (String.IsNullOrEmpty(filename))
                throw new ArgumentException("filename is null or empty.", "filename");

            if (File.Exists(filename) == false)
            {
                throw new FileNotFoundException("Could not find the file.", filename);
            }

            var document = XDocument.Load(filename);

            SetSessionStateConfiguration(document, mode, stateConnectionString, sqlConnectionString, allowCustomSqlDatabase, cookieless, timeout);

            File.SetAttributes(filename, FileAttributes.Normal);

            document.Save(filename);
        }
        
        public static void SetSessionStateConfiguration(XDocument document, 
            string mode, 
            string stateConnectionString, 
            string sqlConnectionString, 
            string allowCustomSqlDatabase, 
            string cookieless, 
            string timeout)
        {
            if (document == null)
            {
                throw new ArgumentNullException("document", "Argument cannot be null.");
            }

            var element = GetSessionStateConfiguration(document);

            if (element == null)
            {
                throw new InvalidOperationException("Session state configuration element was not found.");
            }

            XmlUtility.UpdateValueIfNewValueIsNotNull(element, "mode", mode);
            XmlUtility.UpdateValueIfNewValueIsNotNull(element, "stateConnectionString", stateConnectionString);
            XmlUtility.UpdateValueIfNewValueIsNotNull(element, "sqlConnectionString", sqlConnectionString);
            XmlUtility.UpdateValueIfNewValueIsNotNull(element, "allowCustomSqlDatabase", allowCustomSqlDatabase);
            XmlUtility.UpdateValueIfNewValueIsNotNull(element, "cookieless", cookieless);
            XmlUtility.UpdateValueIfNewValueIsNotNull(element, "timeout", timeout);
        }

        public static XElement GetSessionStateConfiguration(string filename)
        {
            if (String.IsNullOrEmpty(filename))
                throw new ArgumentException("filename is null or empty.", "filename");

            if (File.Exists(filename) == false)
            {
                throw new FileNotFoundException("Could not find the file.", filename);
            }

            var document = XDocument.Load(filename);

            return GetSessionStateConfiguration(document);
        }

        public static XElement GetSessionStateConfiguration(XDocument document)
        {
            if (document == null)
                throw new ArgumentNullException("document", "document is null.");

            XElement systemWeb = GetSystemWeb(document);

            var sessionState = systemWeb.Element("sessionState");

            return sessionState;
        }

        private static XElement GetSystemWeb(XDocument document)
        {
            var root = document.Root;

            if (root == null || root.Name != "configuration")
            {
                throw new InvalidOperationException("Document is not a configuration file.");
            }

            var systemWeb = root.Element("system.web");

            if (systemWeb == null)
            {
                throw new InvalidOperationException("Could not locate the <system.web /> configuration section.");
            }
            return systemWeb;
        }
    }
}
