using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.TeamBuild2012;
using System.IO;
using System.Activities;
using Benday.TeamBuild2012.Activities;

namespace Benday.TeamBuild2012.UnitTests
{
    [TestClass]
    public class MsBuildParametersParserFixture
    {
        public MsBuildParametersParserFixture()
        {
        }

        [TestMethod]
        public void ParseParametersForOneParameter()
        {
            string parameterString = "/p:DeployToEnvironment=Test;";

            IDictionary<string, string> parametersAndValues =
                MsBuildParameterParser.Parse(parameterString);

            Assert.AreEqual<int>(1, parametersAndValues.Count, "Count was wrong.");

            AssertParameterValue(parametersAndValues, "DeployToEnvironment", "Test");
        }

        [TestMethod]
        public void ParseParametersWhenParameterStringDoesNotStartWithSlashPColon()
        {
            string parameterString = "DeployToEnvironment=Test;";

            IDictionary<string, string> parametersAndValues =
                MsBuildParameterParser.Parse(parameterString);

            Assert.AreEqual<int>(1, parametersAndValues.Count, "Count was wrong.");

            AssertParameterValue(parametersAndValues, "DeployToEnvironment", "Test");
        }

        private void AssertParameterValue(
            IDictionary<string, string> parametersAndValues,
            string expectedKey,
            string expectedValue)
        {
            Assert.IsTrue(
                parametersAndValues.ContainsKey(expectedKey),
                "Dictionary did not contain a key named '{0}'.",
                expectedKey);
            Assert.AreEqual<string>(
                expectedValue,
                parametersAndValues[expectedKey],
                "Dictionary did not contain a key named '{0}'.",
                expectedKey);
        }

        [TestMethod]
        public void ParseParametersWhenParameterStringDoesNotEndWithSemicolon()
        {
            string parameterString = "/p:DeployToEnvironment=Test;Key2=Value2";

            IDictionary<string, string> parametersAndValues =
                MsBuildParameterParser.Parse(parameterString);

            Assert.AreEqual<int>(2, parametersAndValues.Count, "Count was wrong.");

            AssertParameterValue(parametersAndValues, "DeployToEnvironment", "Test");
            AssertParameterValue(parametersAndValues, "Key2", "Value2");
        }

        [TestMethod]
        public void ParseParametersWhenParameterStringDoesNotStartWithPColonOrEndWithSemicolon()
        {
            string parameterString = "DeployToEnvironment=Test;Key2=Value2";

            IDictionary<string, string> parametersAndValues =
                MsBuildParameterParser.Parse(parameterString);

            Assert.AreEqual<int>(2, parametersAndValues.Count, "Count was wrong.");

            AssertParameterValue(parametersAndValues, "DeployToEnvironment", "Test");
            AssertParameterValue(parametersAndValues, "Key2", "Value2");
        }

        [TestMethod]
        public void ParseParametersForTwoParameter()
        {
            string parameterString = "/p:DeployToEnvironment=Test;Key2=Value2;";

            IDictionary<string, string> parametersAndValues =
                MsBuildParameterParser.Parse(parameterString);

            Assert.AreEqual<int>(2, parametersAndValues.Count, "Count was wrong.");

            AssertParameterValue(parametersAndValues, "DeployToEnvironment", "Test");
            AssertParameterValue(parametersAndValues, "Key2", "Value2");
        }

        [TestMethod]
        public void ParseParametersForDuplicateParameterNameIgnoresDuplicate()
        {
            string parameterString = "/p:DeployToEnvironment=Test;Key2=Value2;Key2=Duplicate;";

            IDictionary<string, string> parametersAndValues =
                MsBuildParameterParser.Parse(parameterString);

            Assert.AreEqual<int>(2, parametersAndValues.Count, "Count was wrong.");

            AssertParameterValue(parametersAndValues, "DeployToEnvironment", "Test");
            AssertParameterValue(parametersAndValues, "Key2", "Value2");
        }

        [TestMethod]
        public void ParseParametersIgnoresMalformedItems()
        {
            string parameterString = "/p:DeployToEnvironment=Test;Malformed1;Key2=Value2;Malformed2";

            IDictionary<string, string> parametersAndValues =
                MsBuildParameterParser.Parse(parameterString);

            Assert.AreEqual<int>(2, parametersAndValues.Count, "Count was wrong.");

            AssertParameterValue(parametersAndValues, "DeployToEnvironment", "Test");
            AssertParameterValue(parametersAndValues, "Key2", "Value2");
        }

        [TestMethod]
        public void ParseParametersForEmptyStringZeroParameters()
        {
            string parameterString = String.Empty;

            IDictionary<string, string> parametersAndValues =
                MsBuildParameterParser.Parse(parameterString);

            Assert.AreEqual<int>(0, parametersAndValues.Count, "Count was wrong.");
        }

        [TestMethod]
        public void ParseParametersForSlashPColonZeroParameters()
        {
            string parameterString = "/p:";

            IDictionary<string, string> parametersAndValues =
                MsBuildParameterParser.Parse(parameterString);

            Assert.AreEqual<int>(0, parametersAndValues.Count, "Count was wrong.");
        }
    }
}
