using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.TeamBuild2012;
using System.IO;
using System.Activities;

namespace Benday.TeamBuild2012.UnitTests
{
    [TestClass]
    public class TouchFileFixture : UnitTestBase
    {
        public TouchFileFixture()
        {
        }

        protected override void OnTestInitialize()
        {
            if (File.Exists(TargetFilePath) == true)
            {
                File.Delete(TargetFilePath);

                Assert.IsFalse(File.Exists(TargetFilePath), "Target file '{0}' should not exist.", TargetFilePath);
            }
        }        
        
        public string TargetFilePath
        {
            get
            {
                return Path.Combine(TestContext.TestDeploymentDir, "targetFile.txt");
            }
        }

        private void CreateTestFileAndSetDate(string targetFile, DateTime fileDate)
        {
            File.WriteAllText(targetFile, "test file");

            File.SetLastWriteTime(targetFile, fileDate);
        }

        [TestMethod]
        public void TouchFileFixture_CreateTestFileAndSetDate()
        {
            DateTime expected = new DateTime(2005, 8, 1);

            CreateTestFileAndSetDate(TargetFilePath, expected);

            var actual = File.GetLastWriteTime(TargetFilePath);

            Assert.AreEqual<DateTime>(expected, actual, "File write date was wrong.");
        }

        [TestMethod]
        public void TouchFileFixture_TouchFileActivityUpdatesFileDate()
        {
            DateTime startingDateValue = new DateTime(2005, 8, 1);
            CreateTestFileAndSetDate(TargetFilePath, startingDateValue);

            // mark the target as Read Only
            File.SetAttributes(TargetFilePath, FileAttributes.ReadOnly);

            // Create an instance of our test workflow
            var workflow = new WorkflowForTouchFile();

            // Create the workflow run-time environment
            var workflowInvoker = new WorkflowInvoker(workflow);

            // New up an instance of our mock BuildDetail object to add to the Workflow
            // environment's extensions
            BuildDetail buildDetailInstance = new BuildDetail();
            workflowInvoker.Extensions.Add(buildDetailInstance);

            Dictionary<string, object> variables = new Dictionary<string, object>();

            variables.Add("TargetPath", TargetFilePath);

            // Invoke the workflow and capture the outputs
            IDictionary<String, Object> outputs = workflowInvoker.Invoke(variables);


            // verify that the file write date was updated
            Assert.AreNotEqual<DateTime>(startingDateValue, File.GetLastWriteTime(TargetFilePath), "Last write time for target file was not updated.");
        }
    }
}
