using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.TeamBuild2012;
using System.IO;
using System.Xml.Linq;
using Benday.TeamBuild2012.Activities;

namespace Benday.TeamBuild2012.UnitTests
{

    [TestClass]
    public class WebConfigEditorFixture : UnitTestBase
    {
        private string GetPathToTestWebApp()
        {
            string path = TestContext.TestDeploymentDir;

            Assert.IsTrue(Directory.Exists(path), "Path does not exist.  '{0}'", path);

            return path;
        }

        public string TestWebConfig
        {
            get
            {
                string filename = Path.Combine(GetPathToTestWebApp(), "web.config");

                Assert.IsTrue(File.Exists(filename), "Could not find config file at '{0}'.", filename);

                return filename;
            }
        }

        [TestMethod]
        public void GetSessionStateConfiguration()
        {
            string expectedMode = "SQLServer";
            string expectedStateConnectionString = "tcpip=127.0.0.1:42424";
            string expectedSqlConnectionString = @"data source=.\SQLEXPRESS;Integrated Security=SSPI;";
            string expectedAllowCustomSqlDatabase = "true";
            string expectedCookieless = "false";
            string expectedTimeout = "20";

            XElement actual = WebConfigEditor.GetSessionStateConfiguration(TestWebConfig);

            Assert.IsNotNull(actual, "Configuration was null.");

            Assert.AreEqual<string>("sessionState", 
                actual.Name.LocalName, 
                "Node name was wrong.");

            AssertAttributeValue(actual, "mode", expectedMode);
            AssertAttributeValue(actual, "stateConnectionString", expectedStateConnectionString);
            AssertAttributeValue(actual, "sqlConnectionString", expectedSqlConnectionString);
            AssertAttributeValue(actual, "allowCustomSqlDatabase", expectedAllowCustomSqlDatabase);
            AssertAttributeValue(actual, "cookieless", expectedCookieless);
            AssertAttributeValue(actual, "timeout", expectedTimeout);
        }

        [TestMethod]
        public void SetSessionStateConfigurationForAllValues()
        {
            XDocument document = XDocument.Load(TestWebConfig);

            string expectedMode = "mode";
            string expectedStateConnectionString = "state";
            string expectedSqlConnectionString = @"sql";
            string expectedAllowCustomSqlDatabase = "false";
            string expectedCookieless = "true";
            string expectedTimeout = "40";
            
            WebConfigEditor.SetSessionStateConfiguration(document, 
                expectedMode, 
                expectedStateConnectionString, 
                expectedSqlConnectionString, 
                expectedAllowCustomSqlDatabase, 
                expectedCookieless, 
                expectedTimeout);
                
            XElement actual = WebConfigEditor.GetSessionStateConfiguration(document);

            AssertAttributeValue(actual, "mode", expectedMode);
            AssertAttributeValue(actual, "stateConnectionString", expectedStateConnectionString);
            AssertAttributeValue(actual, "sqlConnectionString", expectedSqlConnectionString);
            AssertAttributeValue(actual, "allowCustomSqlDatabase", expectedAllowCustomSqlDatabase);
            AssertAttributeValue(actual, "cookieless", expectedCookieless);
            AssertAttributeValue(actual, "timeout", expectedTimeout);
        }

        [TestMethod]
        public void SetSessionStateConfigurationIgnoresNullValueForTimeout()
        {
            XDocument document = XDocument.Load(TestWebConfig);

            string expectedMode = "mode";
            string expectedStateConnectionString = "state";
            string expectedSqlConnectionString = @"sql";
            string expectedAllowCustomSqlDatabase = "false";
            string expectedCookieless = "true";
            string expectedTimeout = "20";

            WebConfigEditor.SetSessionStateConfiguration(document,
                expectedMode,
                expectedStateConnectionString,
                expectedSqlConnectionString,
                expectedAllowCustomSqlDatabase,
                expectedCookieless,
                null);

            XElement actual = WebConfigEditor.GetSessionStateConfiguration(document);

            AssertAttributeValue(actual, "mode", expectedMode);
            AssertAttributeValue(actual, "stateConnectionString", expectedStateConnectionString);
            AssertAttributeValue(actual, "sqlConnectionString", expectedSqlConnectionString);
            AssertAttributeValue(actual, "allowCustomSqlDatabase", expectedAllowCustomSqlDatabase);
            AssertAttributeValue(actual, "cookieless", expectedCookieless);
            AssertAttributeValue(actual, "timeout", expectedTimeout);
        }        
    }
}
